<?php

namespace Drupal\Tests\site_alert\Kernel;

use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\KernelTests\KernelTestBase;

/**
 * Base class for kernel tests for the Site Alerts module.
 *
 * @group site_alert
 */
abstract class SiteAlertKernelTestBase extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'datetime_range',
    'options',
    'site_alert',
    'text',
  ];

  /**
   * Test site alert entities.
   *
   * @var \Drupal\site_alert\Entity\SiteAlert[]
   */
  protected $alerts;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('site_alert');
    $this->entityTypeManager = $this->container->get('entity_type.manager');
  }

  /**
   * Creates site alert entities for the given identifiers.
   *
   * @param array $alerts_to_create
   *   An array of identifiers indicating which site alerts to create. Each
   *   identifier starts with the type of alert to create (one of 'active',
   *   'inactive', 'past', 'current' or 'future'), a hyphen, whether the alert
   *   is active, and optionally ending in a number if multiple instances of the
   *   same type should be created.
   */
  protected function createAlerts(array $alerts_to_create): void {
    $storage = $this->entityTypeManager->getStorage('site_alert');

    foreach ($alerts_to_create as $identifier) {
      preg_match('/^([a-z]+)-([a-z]+)\d*/', $identifier, $matches);

      $values = [
        'active' => $matches[2] === 'active',
        'label' => $this->randomString(),
        'severity' => 'medium',
        'message' => $this->randomString(),
      ];

      switch ($matches[1]) {
        case 'past':
          $values['scheduling'] = [
            'value' => $this->getDateTime('-3 hours'),
            'end_value' => $this->getDateTime('-2 hours'),
          ];
          break;

        case 'present':
          $values['scheduling'] = [
            'value' => $this->getDateTime('-1 hours'),
            'end_value' => $this->getDateTime('+1 hours'),
          ];
          break;

        case 'future':
          $values['scheduling'] = [
            'value' => $this->getDateTime('+2 hours'),
            'end_value' => $this->getDateTime('+3 hours'),
          ];
          break;
      }

      $this->alerts[$identifier] = $storage->create($values);
      $this->alerts[$identifier]->save();
    }
  }

  /**
   * Returns a string formatted date/time suitable for DateRangeItem field.
   *
   * @param string $time_diff
   *   A string representation of date/time as a difference.
   *
   * @return string
   *   A string formatted date/time suitable for DateRangeItem field.
   */
  protected function getDateTime(string $time_diff): string {
    $datetime = new \DateTime($time_diff, new \DateTimeZone(DateTimeItemInterface::STORAGE_TIMEZONE));
    $timestamp = $datetime->getTimestamp();
    return $this->container->get('date.formatter')
      ->format(
        $timestamp,
        'custom',
        DateTimeItemInterface::DATETIME_STORAGE_FORMAT,
        DateTimeItemInterface::STORAGE_TIMEZONE
      );
  }

}
